#!/usr/bin/python

import os
import re
import sys
import shutil
import tempfile
import socket
from shlex import split
import subprocess
from subprocess import (
    call,
    check_call,
    check_output,
    Popen,
    CalledProcessError
)

sys.path.insert(0, os.path.join(os.environ['CHARM_DIR'], 'lib'))

from charmhelpers import fetch
from charmhelpers.core import (
    hookenv,
)
log = hookenv.log

GPFS_PATH = '/Linux/x86_64'


def add_to_path(p, new):
    return p if new in p.split(':') else p + ':' + new

os.environ['PATH'] = add_to_path(os.environ['PATH'], '/usr/lpp/mmfs/bin')


## installation methods ##
def _get_source(url, source_dir=""):
    return "deb %s%s ./" % (url, source_dir)


def check_empty_source(gpfs_url):
    # Check if there is a repository to install from
    # TODO: check if packages are already installed
    if not gpfs_url:
        log("No GPFS source available. Use juju set gpfs gpfs_url=<url> "
            "to install packages. See README for more info.", log.ERROR)
        sys.exit(0)


def gpfs_install(url, version):
    # install main gpfs packages
    gpfs_source = _get_source(url, source_dir=GPFS_PATH)
    log('Gpfs source' + gpfs_source)

    # Add gpfs source and update apt
    fetch.add_source(gpfs_source)
    try:
        fetch.apt_update(fatal=True)
        # Install gpfs packages
        fetch.apt_install("gpfs*", options=["--allow-unauthenticated"],
                          fatal=True)
    except CalledProcessError:
        #TODO: verify source was configured correctly
        raise


def gpfs_install_updates(url, version, update=True):
    # install update packages (patches), if given
    if update:
        log('Applying patches')
        source_dir = '/gpfs' + version + '_patches' + GPFS_PATH
        patch_source = _get_source(url, source_dir=source_dir)
        fetch.add_source(patch_source)
        fetch.apt_update(fatal=True)
        fetch.apt_install("gpfs*", options=["--allow-unauthenticated"],
                          fatal=True)


## ssh key methods ##
def create_ssh_keys():
    # Generate ssh keys if needed
    log("Configuring ssh keys")
    if not os.path.isfile("/root/.ssh/id_rsa"):
        call(split('ssh-keygen -q -N "" -f /root/.ssh/id_rsa'))
        # Ensure permissions are good
        os.chmod('/root/.ssh/id_rsa.pub', 0600)
        os.chmod('/root/.ssh/id_rsa', 0600)


def get_ssh_keys():
    # get public and private ssh keys
    with open("/root/.ssh/id_rsa.pub", "r") as idfile:
        pubkey = idfile.read()
        idfile.close()
    with open("/root/.ssh/id_rsa", "r") as idfile:
        privkey = idfile.read()
        idfile.close()
    with open("/root/.ssh/authorized_keys", "w+") as idfile:
        idfile.write(pubkey)
        idfile.close()
    return [privkey, pubkey]


def set_ssh_key(key, private=False):
    if key:
        if private is False:
            f = open("/root/.ssh/id_rsa.pub", "w")
            f.write(key)
            f.close()
            os.chmod('/root/.ssh/id_rsa.pub', 0600)

            with open("/root/.ssh/authorized_keys", "w+") as idfile:
                idfile.write(key)
                idfile.close()
        else:
            f = open("/root/.ssh/id_rsa", "w")
            f.write(key)
            f.close()
            os.chmod('/root/.ssh/id_rsa', 0600)


def configure_ssh():
    # Configure sshd_config file to allow root
    sshconf = open("/etc/ssh/sshd_config", 'r')
    tf = tempfile.NamedTemporaryFile(mode='w+t', delete=False)
    tfn = tf.name
    for line in sshconf:
        if not line.startswith("#"):
            if "PermitRootLogin" in line:
                tf.write("# Updated by GPFS charm: ")
                tf.write(line)
            else:
                tf.write(line)
        else:
            tf.write(line)
    tf.write("# added by GPFS charm:\n")
    tf.write("PermitRootLogin without-password\n")
    sshconf.close()
    tf.close()
    shutil.copy(tfn, '/etc/ssh/sshd_config')
    call(split('service ssh reload'))

    # Avoid the host key confirmation
    with open("/root/.ssh/config", "w+") as idfile:
        idfile.write("StrictHostKeyChecking no\n")
        idfile.close()


## Other utility methods##
def validate_quorum(quorum):
    valid_values = ['', 'quorum', 'non-quorum']
    return quorum if quorum in valid_values else ''


## cluster management methods ##
def build_modules():
# build binary gpfs modules after installation
    try:
        check_call(["mmbuildgpl"])
    except CalledProcessError:
        log('mmbuildgpl was not executed', level=hookenv.WARNING)
    except OSError:
        log('mmbuildgpl not found/installed')


def start():
    # Only manager can start/stop cluster
    if is_manager():
        log(check_output(split('mmstartup -a')))


def stop():
    # stop the server
    if is_manager():
        check_output(split('mmshutdown -a'))


def node_exists(nodename):
    # Check if node has already been added to cluster
    lscluster = check_output('mmlscluster')
    node = re.search('^ *\d+.*%s.*$' % nodename, lscluster, re.M)
    return False if node is None else True


def add_node(nodename, m_designation='client', q_designation='nonquorum'):
    # add new node to the cluster
    if cluster_exists() and not node_exists(nodename):
        log(check_output(split('mmaddnode -N %s:%s-%s' %
                         (nodename, m_designation, q_designation))))
    else:
        log('Node %s could not be added. Manager not defined' % nodename)


def delete_node(nodename):
    if node_exists(nodename):
        log('Trying to delete node %s:' % nodename)
        log(check_output(split('mmdelnode -N %s' % nodename)))


def apply_license(nodename):
    check_output(split('mmchlicense server --accept -N %s' % nodename))


def set_manager_file():
    # define a file that indicates the node is a manager
    open("/var/mmfs/.manager", "a").close()


def is_manager():
    # check if the node is a manager
    if os.path.exists("/var/mmfs/.manager"):
        return True


def cluster_exists():
    # Check if cluster is already defined
    return True if call('mmlscluster') == 0 else False


def create_cluster(hostname):
    # create the cluster for the hostname, manager only
    if is_manager() and not cluster_exists():
        log(check_output(split('mmcrcluster -N %s:quorum-manager' %
                         hostname + ' -r /usr/bin/ssh -R /usr/bin/scp')))


def chk_dns(hostname, ip):
    try:
        socket.gethostbyname(hostname)
    except:
        log("Hostname not resolving, adding to /etc/hosts")
        with open("/etc/hosts", "a") as hostfile:
            hostfile.write("%s %s\n" % (ip, hostname))
            hostfile.close()
